﻿using System;
using System.IO;
using System.Text;
using FindReplace;

namespace FO3SE
{
    public class Fallout3
    {
        private const byte BAR = 124;

        private SaveInfo _Info;
        public SaveInfo Info
        {
            get { return _Info; }
        }

        private int _Special;
        private int _Level;

        public int SaveNumber
        {
            get { return _Header.SaveNumber; }
            set { _Header.SaveNumber = value; }
        }
        public string Time
        {
            get { return _Header.Time; }
            set { _Header.Time = value; }
        }
        public short TimeLength
        {
            get { return _Header.TimeLength; }
        }
        private Header _Header;
        public void WriteHeader()
        {
            _Header.Write(_Info.Path);
        }

        public void FindOffsets()
        {
            LargeFile file = new LargeFile(_Info.Path);
            int[] results = file.Find(_Info.Player);
            int index = results[results.Length - 1];

            // The easy one...
            _Special = index - 11;

            // The hard one...
            FileStream fs = new FileStream(_Info.Path, FileMode.Open);
            BinaryReader br = new BinaryReader(fs);
            fs.Seek(index - 256, SeekOrigin.Begin);
            byte[] read = br.ReadBytes(256);
            br.Close();
            fs.Close();

            byte count = 0;
            for (byte i = 255; i > 0; i--)
            {
                if (read[i] == 124)
                {
                    if (count >= 35)
                    {
                        _Level = index - 255 + i + count - 16;
                        break;
                    }
                    count = 0;
                }
                else
                    count++;
            }
        }

        public byte[] ReadSpecial()
        {
            FileStream fs = new FileStream(_Info.Path, FileMode.Open);
            BinaryReader br = new BinaryReader(fs);
            fs.Seek(_Special, SeekOrigin.Begin);
            byte[] output = br.ReadBytes(7);
            br.Close();
            fs.Close();
            return output;
        }
        public void WriteSpecial(byte[] values)
        {
            FileStream fs = new FileStream(_Info.Path, FileMode.Open);
            BinaryWriter bw = new BinaryWriter(fs);
            fs.Seek(_Special, SeekOrigin.Begin);
            bw.Write(values);
            bw.Flush();
            bw.Close();
            fs.Close();
        }

        public byte ReadLevel()
        {
            FileStream fs = new FileStream(_Info.Path, FileMode.Open);
            BinaryReader br = new BinaryReader(fs);
            fs.Seek(_Level, SeekOrigin.Begin);
            byte output = br.ReadByte();
            br.Close();
            fs.Close();
            return output;
        }
        public void WriteLevel(byte value)
        {
            if (_Level != 0)
            {
                FileStream fs = new FileStream(_Info.Path, FileMode.Open);
                BinaryWriter bw = new BinaryWriter(fs);
                fs.Seek(_Level, SeekOrigin.Begin);
                bw.Write(value);
                bw.Flush();
                bw.Close();
                fs.Close();
            }
        }

        public void UpdateSaveInfo()
        {
            _Info.Number = _Header.SaveNumber;
            _Info.PlayTime = _Header.Time.TrimStart('0');
            if (_Info.PlayTime[0] == '.')
                _Info.PlayTime = '0' + _Info.PlayTime;

            _Info.UpdatePath();
        }
        public void Backup()
        {
            if (_Info.Source == _Info.Path)
                File.Copy(_Info.Source, _Info.Path + ".bak", true);
            else
                File.Copy(_Info.Source, _Info.Path, true);
        }
        public bool IsTutorial()
        {
            bool output = true;
            FileStream fs = new FileStream(_Info.Path, FileMode.Open);
            BinaryReader br = new BinaryReader(fs);
            fs.Seek(_Special - 1, SeekOrigin.Begin);
            if (br.ReadChar() == '|')
                output = false;
            br.Close();
            fs.Close();
            return output;
        }
        public bool IsBrokenSteel()
        {
            LargeFile file = new LargeFile(_Info.Path);
            int[] results = file.Find("BrokenSteel.esm", 4096, 1);
            if (results.Length > 0)
                return true;
            else
                return false;
        }

        public Fallout3(string file)
        {
            _Info = new SaveInfo(file);
            _Header = new Header(_Info.Path);
            FindOffsets();
        }

        public struct SaveInfo
        {
            private string _Source;
            public string Source
            {
                get { return _Source; }
            }

            private string _Path;
            public string Path
            {
                get { return _Path; }
                set
                {
                    _Source = _Path;
                    _Path = value;
                }
            }

            public int Number;
            public string Player;
            public string Location;
            public string PlayTime;

            public SaveInfo(string path)
            {
                _Path = path;
                _Source = path;

                FileInfo fi = new FileInfo(path);
                string name = fi.Name.Substring(0, fi.Name.Length - 4);

                string temp = name.Substring(0, name.IndexOf('-') - 1);
                string[] parts = temp.Split(' ');
                Number = Convert.ToInt32(parts[1]);

                temp = name.Substring(name.IndexOf('-') + 1);
                parts = temp.Split(',');
                Player = parts[0].Trim(' ');
                Location = parts[1].Trim(' ');
                PlayTime = parts[2].Trim(' ');
            }

            public void UpdatePath()
            {
                FileInfo fi = new FileInfo(Path);
                Path = fi.DirectoryName + "\\Save " + Number + " - " + Player + ", " + Location + ", " + PlayTime + ".fos";
            }
            public string GetName()
            {
                FileInfo fi = new FileInfo(_Path);
                return fi.Name;
            }
            public string GetNameWithoutExtension()
            {
                FileInfo fi = new FileInfo(_Path);
                return fi.Name.Substring(0, fi.Name.Length - fi.Extension.Length);
            }
        }

        public struct Header
        {
            public const string IDENTIFIER = "FO3SAVEGAME";
            public int HeaderLength;
            public int Unknown2;
            public int Unknown3;
            public int Unknown4;
            public int SaveNumber;
            public short NameLength;
            public string Name;
            public short KarmaLength;
            public string Karma;
            public int Level; // Read Only
            public short LocationLength;
            public string Location;
            public short TimeLength;
            public string Time;

            public Header(string path)
            {
                FileStream fs = new FileStream(path, FileMode.Open, FileAccess.Read);
                BinaryReader br = new BinaryReader(fs);

                byte[] bytes = br.ReadBytes(Header.IDENTIFIER.Length);
                if (Header.IDENTIFIER != Encoding.ASCII.GetString(bytes))
                    throw new Exception("Not a valid Fallout Save file.");

                HeaderLength = br.ReadInt32();
                Unknown2 = br.ReadInt32();

                fs.Position++; // skip |
                Unknown3 = br.ReadInt32();

                fs.Position++;
                Unknown4 = br.ReadInt32();

                fs.Position++;
                SaveNumber = br.ReadInt32();

                fs.Position++;
                NameLength = br.ReadInt16();

                fs.Position++;
                Name = Encoding.ASCII.GetString(br.ReadBytes(NameLength));

                fs.Position++;
                KarmaLength = br.ReadInt16();

                fs.Position++;
                Karma = Encoding.ASCII.GetString(br.ReadBytes(KarmaLength));

                fs.Position++;
                Level = br.ReadInt32();

                fs.Position++;
                LocationLength = br.ReadInt16();

                fs.Position++;
                Location = Encoding.ASCII.GetString(br.ReadBytes(LocationLength));

                fs.Position++;
                TimeLength = br.ReadInt16();

                fs.Position++;
                Time = Encoding.ASCII.GetString(br.ReadBytes(TimeLength));

                br.Close();
                fs.Close();
            }

            public void Write(string path)
            {
                if (NameLength != Name.Length)
                    throw new Exception("Name length must not be changed.");

                if (KarmaLength != Karma.Length)
                    throw new Exception("Karma length must not be changed.");

                if (LocationLength != Location.Length)
                    throw new Exception("Location length must not be changed.");

                if (TimeLength != Time.Length)
                    throw new Exception("Time length must not be changed.");

                FileStream fs = new FileStream(path, FileMode.Open, FileAccess.Write);
                BinaryWriter bw = new BinaryWriter(fs);

                fs.Seek(0, SeekOrigin.Begin);
                bw.Write(Encoding.ASCII.GetBytes(Header.IDENTIFIER));
                bw.Write(HeaderLength);
                bw.Write(Unknown2);
                bw.Write(BAR);
                bw.Write(Unknown3);
                bw.Write(BAR);
                bw.Write(Unknown4);
                bw.Write(BAR);
                bw.Write(SaveNumber);
                bw.Write(BAR);
                bw.Write(NameLength);
                bw.Write(BAR);
                bw.Write(Encoding.ASCII.GetBytes(Name));
                bw.Write(BAR);
                bw.Write(KarmaLength);
                bw.Write(BAR);
                bw.Write(Encoding.ASCII.GetBytes(Karma));
                bw.Write(BAR);
                bw.Write(Level);
                bw.Write(BAR);
                bw.Write(LocationLength);
                bw.Write(BAR);
                bw.Write(Encoding.ASCII.GetBytes(Location));
                bw.Write(BAR);
                bw.Write(TimeLength);
                bw.Write(BAR);
                bw.Write(Encoding.ASCII.GetBytes(Time));
                bw.Flush();

                bw.Close();
                fs.Close();
            }
        }
    }
}
