using FindReplace.Interfaces;


/*
 * Copyright 2007 Lee Glasser
 * 
 * This code is distributed under the terms of the GNU General Public License
 * (GPL).
 * 
 * This file is part of the Includes package.
 * 
 * "Includes.FindReplace" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * "Includes.FindReplace" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with "Includes.FindReplace."  If not, see <http://www.gnu.org/licenses/>.
 */


namespace FindReplace.Types
{
    /// <summary>
    /// Strongly typed (Point) class derived from IFound used for returning 2D results.
    /// </summary>
    public class Found2D : IFound<Point>
    {
        private Point _Origin;
        public Point Origin
        {
            get { return _Origin; }
            set { _Origin = value; }
        }

        private Point _End;
        public Point End
        {
            get { return _End; }
            set { _End = value; }
        }

        public Found2D()
        {
        }
        public Found2D(Point origin, Point end)
        {
            _Origin = origin;
            _End = end;
        }

        /// <summary>
        /// Converts the contained data into a string.
        /// </summary>
        /// <returns>Zero-based: "(ox, oy) (ex, ey)"</returns>
        public new string ToString()
        {
            return "(" + _Origin.X + ", " + _Origin.Y + ") (" + _End.X + ", " + _End.Y + ")";
        }
        /// <summary>
        /// Converts the contained data into a string.
        /// </summary>
        /// <returns>One-based: "(ox, oy) (ex, ey)"</returns>
        public string ToStringCounting()
        {
            return "(" + (_Origin.X + 1) + ", " + (_Origin.Y + 1) + ") (" + (_End.X + 1) + ", " + (_End.Y + 1) + ")";
        }
    }
}
