using FindReplace.Interfaces;


/*
 * Copyright 2007 Lee Glasser
 * 
 * This code is distributed under the terms of the GNU General Public License
 * (GPL).
 * 
 * This file is part of the Includes package.
 * 
 * "Includes.FindReplace" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * "Includes.FindReplace" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with "Includes.FindReplace."  If not, see <http://www.gnu.org/licenses/>.
 */


namespace FindReplace.Types
{
    /// <summary>
    /// Strongly typed (Point3D) class derived from IFound used for returning 3D results.
    /// </summary>
    public class Found3D : IFound<Point3D>
    {
        private Point3D _Origin;
        public Point3D Origin
        {
            get { return _Origin; }
            set { _Origin = value; }
        }

        private Point3D _End;
        public Point3D End
        {
            get { return _End; }
            set { _End = value; }
        }
        
        public Found3D()
        {
        }
        public Found3D(Point3D origin, Point3D end)
        {
            _Origin = origin;
            _End = end;
        }

        /// <summary>
        /// Converts the contained data into a string.
        /// </summary>
        /// <returns>Zero-based: "(ox, oy, oz) (ex, ey, ez)"</returns>
        public new string ToString()
        {
            return "(" + _Origin.X + ", " + _Origin.Y + ", " + _Origin.Z + ") (" + _End.X + ", " + _End.Y + ", " + _End.Z + ")";
        }
        /// <summary>
        /// Converts the contained data into a string.
        /// </summary>
        /// <returns>One-based: "(ox, oy, oz) (ex, ey, ez)"</returns>
        public string ToStringCounting()
        {
            return "(" + (_Origin.X + 1) + ", " + (_Origin.Y + 1) + ", " + (_Origin.Z + 1) + ") (" + (_End.X + 1) + ", " + (_End.Y + 1) + ", " + (_End.Z + 1) + ")";
        }
    }
}
