﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace FO3SE
{
    public partial class Main : Form
    {
        private Fallout3 _FO3;

        public Main()
        {
            InitializeComponent();
            txtSaveFolder.Text = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + @"\My Games\Fallout3\Saves";
        }

        private void SpecialScroll(object sender, EventArgs e)
        {
            TrackBar s = (TrackBar)sender;
            TextBox t = (TextBox)tlpSpecial.Controls["txt" + s.Name.Substring(3)];
            t.Text = s.Value.ToString();
        }
        private void LevelScroll(object sender, EventArgs e)
        {
            txtLevel.Text = trkLevel.Value.ToString();
        }

        private void txtSaveFolder_TextChanged(object sender, EventArgs e)
        {
            if (Directory.Exists(txtSaveFolder.Text))
            {
                cmbFile.Items.Clear();
                foreach (string file in Directory.GetFiles(txtSaveFolder.Text, "*.fos", SearchOption.TopDirectoryOnly))
                {
                    FileInfo fi = new FileInfo(file);
                    if ((fi.Name != "autosave.fos") && (fi.Name != "quicksave.fos"))
                        cmbFile.Items.Add(fi.Name.Substring(0, fi.Name.Length - 4));
                }
            }
        }

        private void cmbFile_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                _FO3 = new Fallout3(txtSaveFolder.Text + "\\" + cmbFile.Items[cmbFile.SelectedIndex] + ".fos");
                bool good = true;
                if (_FO3.Info.Player.Length < 3)
                {
                    DialogResult result = MessageBox.Show("The player name needs to be at least 3 charaters in length to use this application.  You can change your name by opening the console via the tlide key (~), enter the \"ShowNameMenu\" command, and press enter.  Once you have saved your changed name, use this application on the new save.\n\nWould you like to continue anyway?", "Name too short, continue?", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2);
                    if (result == DialogResult.No)
                    {
                        good = false;
                        cmbFile.SelectedText = "";
                    }
                }

                if (good)
                {
                    txtPlayTime.Text = _FO3.Time;
                    txtSaveNum.Text = _FO3.SaveNumber.ToString();

                    byte[] special = _FO3.ReadSpecial();
                    bool bad = false;
                    for (byte i = 0; i < special.Length; i++)
                    {
                        if ((special[i] < 1) || (special[i] > 10))
                            bad = true;
                    }

                    if (bad)
                    {
                        for (byte i = 0; i < special.Length; i++)
                        {
                            TrackBar trk = (TrackBar)tlpSpecial.Controls["trkSpecial" + i];
                            trk.Enabled = false;
                        }
                    }
                    else
                    {
                        for (byte i = 0; i < special.Length; i++)
                        {
                            TrackBar trk = (TrackBar)tlpSpecial.Controls["trkSpecial" + i];
                            trk.Value = special[i];
                            trk.Enabled = true;
                            TextBox txt = (TextBox)tlpSpecial.Controls["txtSpecial" + i];
                            txt.Text = special[i].ToString();
                        }
                    }

                    bool brokensteel = _FO3.IsBrokenSteel();
                    if (brokensteel)
                        trkLevel.Maximum = 30;
                    else
                        trkLevel.Maximum = 20;

                    if (_FO3.IsTutorial())
                        trkLevel.Enabled = false;
                    else
                    {
                        byte level = _FO3.ReadLevel();
                        if ((level < 1) || (level > trkLevel.Maximum))
                            trkLevel.Enabled = false;
                        else
                        {
                            trkLevel.Enabled = true;
                            trkLevel.Value = level;
                            txtLevel.Text = level.ToString();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                ssStatus.Text = ex.Message;
            }
        }

        private void btnLevel_Click(object sender, EventArgs e)
        {
            if (trkLevel.Value != 0)
                trkLevel.Value--;
            txtLevel.Text = trkLevel.Value.ToString();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (cmbFile.Text.Length > 0)
                {
                    cmbFile.BackColor = Color.White;

                    bool good = true;
                    if (txtSaveNum.BackColor == Color.Pink)
                    {
                        MessageBox.Show("The save number entered is invalid.  Make sure it only has numbers.", "Save Number Invalid", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        good = false;
                    }
                    else
                        _FO3.SaveNumber = Convert.ToInt32(txtSaveNum.Text);

                    if (txtPlayTime.BackColor == Color.Pink)
                    {
                        MessageBox.Show("The play time entered is invalid.  It must be exactly " + _FO3.TimeLength + " characters in length.", "Play Time Invalid", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        good = false;
                    }
                    else
                        _FO3.Time = txtPlayTime.Text;

                    if (good)
                    {
                        _FO3.UpdateSaveInfo();

                        _FO3.Backup();
                        _FO3.WriteHeader();
                        byte[] special = new byte[7];
                        for (byte i = 0; i < special.Length; i++)
                        {
                            TextBox txt = (TextBox)tlpSpecial.Controls["txtSpecial" + i];
                            special[i] = Convert.ToByte(txt.Text);
                        }

                        bool bad = false;
                        for (byte i = 0; i < special.Length; i++)
                        {
                            if ((special[i] < 1) || (special[i] > 10))
                                bad = true;
                        }

                        if (!bad)
                            _FO3.WriteSpecial(special);

                        byte level = Convert.ToByte(txtLevel.Text);
                        if ((level >= 1) || (level <= 20))
                            _FO3.WriteLevel(level);

                        txtSaveFolder_TextChanged(this, new EventArgs());
                        for (int i = 0; i < cmbFile.Items.Count; i++)
                        {
                            if (cmbFile.Items[i].ToString() == _FO3.Info.GetNameWithoutExtension())
                                cmbFile.SelectedIndex = i;
                        }
                        ssStatus.Text = "Save successful.";
                    }
                }
                else
                    cmbFile.BackColor = Color.Pink;
            }
            catch (Exception ex)
            {
                ssStatus.Text = ex.Message;
            }
        }

        private void btnSaveFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dlg = new FolderBrowserDialog();
            dlg.SelectedPath = txtSaveFolder.Text;
            dlg.ShowNewFolderButton = false;
            dlg.ShowDialog();
            txtSaveFolder.Text = dlg.SelectedPath;
        }

        private void txtPlayTime_TextChanged(object sender, EventArgs e)
        {
            if (txtPlayTime.Text.Length != _FO3.TimeLength)
                txtPlayTime.BackColor = Color.Pink;
            else
                txtPlayTime.BackColor = Color.White;
        }

        private void txtSaveNum_TextChanged(object sender, EventArgs e)
        {
            try
            {
                Convert.ToInt32(txtSaveNum.Text);
                txtSaveNum.BackColor = Color.White;
            }
            catch { txtSaveNum.BackColor = Color.Pink; }
        }
    }
}
